@@ -1,5 +1,5 @@
 package Mojolicious::Command::export;
-our $VERSION = '0.002';
+our $VERSION = '0.003';
 # ABSTRACT: Export a Mojolicious website to static files
 
 #pod =head1 SYNOPSIS
@@ -45,7 +45,7 @@ our $VERSION = '0.002';
 
 use Mojo::Base 'Mojolicious::Command';
 use Mojo::File qw( path );
-use Mojo::Util qw( getopt );
+use Mojo::Util qw( getopt encode decode );
 
 has description => 'Export site to static files';
 has usage => sub { shift->extract_usage };
@@ -77,12 +77,16 @@ sub run {
     my $ua = Mojo::UserAgent->new;
     $ua->server->app( $self->app );
 
-    my %exported;
+    # A hash of path => knowledge about the path
+    #   link_from => a hash of path -> array of DOM elements linking to original path
+    #   res => The response from the request for this page
+    #   redirect_to => The redirect location, if it was a redirect
+    my %history;
+
     while ( my $page = shift @pages ) {
-        next if $exported{ $page };
-        $exported{ $page }++;
+        next if $history{ $page }{ res };
         my $tx = $ua->get( $page );
-        my $res = $tx->res;
+        my $res = $history{ $page }{ res } = $tx->res;
 
         # Do not try to write error messages
         if ( $res->is_error ) {
@@ -93,6 +97,21 @@ sub run {
             next;
         }
 
+        # Rewrite links to redirects
+        if ( $res->is_redirect ) {
+            my $loc = $history{ $page }{ redirect_to } = $res->headers->location;
+            say "  [redir] Found redirect. Fixing links to this page"
+                unless $self->quiet;
+            for my $link_from ( keys %{ $history{ $page }{ link_from } } ) {
+                for my $el ( @{ $history{ $page }{ link_from }{ $link_from } } ) {
+                    $el->attr( href => $loc );
+                }
+                my $content = $history{ $link_from }{ res }->dom;
+                $self->_write( $root, $link_from, $content );
+            }
+            next;
+        }
+
         my $type = $res->headers->content_type;
         my $content = $tx->res->body;
         if ( $type and $type =~ m{^text/html} and my $dom = $res->dom ) {
@@ -103,10 +122,23 @@ sub run {
 
                     # Don't analyze full URLs
                     next if $url =~ m{^(?:[a-zA-Z]+:)?//};
+                    # Don't analyze in-page fragments
+                    next if $url =~ m{^#};
 
                     # Fix relative paths
                     my $path = $url =~ m{^/} ? $url : $dir->child( $url )."";
-                    if ( !$exported{ $path } ) { # Prune duplicates
+                    # Remove fragment
+                    $path =~ s/#.+//;
+
+                    if ( my $loc = $history{ $path }{ redirect_to } ) {
+                        $el->attr( $attr => $loc );
+                        next;
+                    }
+                    else {
+                        push @{ $history{ $path }{ link_from }{ $page } }, $el;
+                    }
+
+                    if ( !$history{ $path }{ res } ) {
                         push @pages, $path;
                     }
 
@@ -120,18 +152,23 @@ sub run {
             $content = $dom;
         }
 
-        my $to = $root->child( $page );
-        if ( $to !~ m{[.][^/.]+$} ) {
-            $to = $to->child( 'index.html' );
-        }
-        if ( -e $to ) {
-            say "  [delet] $to" unless $self->quiet;
-            unlink $to;
-        }
-        $self->write_file( $to, $content );
+        $self->_write( $root, $page, $content );
     }
 }
 
+sub _write {
+    my ( $self, $root, $page, $content ) = @_;
+    my $to = $root->child( $page );
+    if ( $to !~ m{[.][^/.]+$} ) {
+        $to = $to->child( 'index.html' );
+    }
+    if ( -e $to ) {
+        say "  [delet] $to" unless $self->quiet;
+        unlink $to;
+    }
+    $self->write_file( $to, $content );
+}
+
 1;
 
 __END__
@@ -144,7 +181,7 @@ Mojolicious::Command::export - Export a Mojolicious website to static files
 
 =head1 VERSION
 
-version 0.002
+version 0.003
 
 =head1 SYNOPSIS
 
